/**
* Cette classe permet de grer les arbres utiles pour le codage de Huffman
* @author Anya Desilles
* @version 1.0
*/
public class ArbreCodage {
	/**
	* la chane de caractres associe  la racine
	*/
	private String racine;
	
	/**
	* le poids de la racine (probabilit)
	*/
	private double poids;
	
	/** 
	* Un lien vers un ventuel arbre  gauche
	*/
	private ArbreCodage gauche;
	
	/** 
	* Un lien vers un ventuel arbre  droite
	*/
	private ArbreCodage droite;
	

	/**
	* Un constructeur vide
	*/
	public ArbreCodage() {
		this.racine = new String("");
		this.poids = 0;
		this.gauche=null;
		this.droite=null;
		}
	
	/**
	* On construit une feuille ou noeud externe
	* @param racine : la chane associe  cette feuille
	* @param poids : le poids de cette feuille  
	*/
	public ArbreCodage(String racine, double poids){
		this.racine=racine;
		this.poids=poids;
		this.gauche=null;
		this.droite=null;
	}
	
	
	/**
	* On construit un noeud interne
	* @param racine : la chane associe  ce noeud interne
	* @param poids : le poids de ce noeud interne
	* @param gauche : l'arbre  gauche de ce noeud
	* @param droite : l'arbre  droite de ce noeud 
	*/
	public ArbreCodage(String racine,ArbreCodage gauche, ArbreCodage droite,double poids){
		this.racine=racine;
		this.poids=poids;
		this.droite=droite;
		this.gauche=gauche;
		}
	
	/*
	* @return : le poids de la racine de l'arbre
	*/
	public double getPoids() {
		return this.poids;
	}
	
	/*
	* @return : l'arbre gauche de l'arbre
	*/
	public ArbreCodage getGauche(){
		return this.gauche;
	}
	
	/*
	* @return : l'arbre gauche de l'arbre
	*/
	public ArbreCodage getDroit(){
		return this.droite;
	}
	
	/*
	* @return : l'arbre gauche de l'arbre
	*/
	public String getLettre(){
		return this.racine;
	}
	
	/**
	* @return : <b>true</b> si l'arbre est une feuille <b>false</b> sinon
	*/
	public  boolean estFeuille(){
		if((this.gauche==null) && (this.droite==null)) {
			return true;
		}
		else return false;
	}
	
	/**
	 * Une mthode qui parcourt l'arbre en profondeur en affichant 
	 * les codes binaires de tous les chenims entre la racine
	 * et les  feuilles. 
  */
	public void printBin(){
		printBinR("");
	}
	
	/**
	 * Une mthode qui parcourt l'arbre en profondeur en affichant 
	 * les codes binaires de tous les chenims entres la racine
	 * et les  feuilles. 
	 * @param mot : le code binaire des noeuds anctres
  */
	private void printBinR(String mot) {
		if (this.estFeuille()) {
			System.out.println(this.getLettre()+" "+mot);}
		else {
			
			this.gauche.printBinR(mot+"1");
					this.droite.printBinR(mot+"0");
		}
	}	
}



