/**
* Cette classe permet de grer un fort d'arbres utiles pour le codage de Huffman.
* Cette fort est une liste d'arbres ordonne par ordre dcroissant de leur poids.
* @author Anya Desilles
* @version 1.0
*/

import java.util.*;

public class ForetCodage {
	/**
	* La liste de d'arbres
	*/
	Vector<ArbreCodage> mesArbres;
	
	/**
	* Constructeur dans le cas d'une liste vide
	*/
	public ForetCodage() {
		mesArbres = new Vector<ArbreCodage>();
	}

	/**
	* On ajoute un nouvel arbre dans la liste en respectant l'ordre des poids
	* @param ar : l'arbre  insrer
	*/
	public void ajouterArbre(ArbreCodage ar) {
		if(mesArbres.size() == 0) { // si  la liste est vide
			mesArbres.add(ar);      // l'lment ajout devient le premier et le dernier
		}
		else {
			/*
			 * Sinon on parcours la liste  partir du dbut
			 * pour trouver le premier lment dont le poids est infrieur au poids 
			 * de l'arbre  ajouter.
			 * l'arbre sera alors ajout juste avant l'lment trouv.
			 * 
			 */
			double p=ar.getPoids();   
			int i = 0;
			int nbArbres = mesArbres.size();
			
			while(i < nbArbres && mesArbres.get(i).getPoids() > p) i++;
			mesArbres.insertElementAt(ar,i);
		}
	}

	/**
	* Initialisation d'une nouvelle fort (ordonne) avec un tableau d'arbres
	* @param tab : le tableau d'arbres
	*/
	public ForetCodage(ArbreCodage [] tab) {
		mesArbres = new Vector<ArbreCodage>();
		
		for(ArbreCodage a : tab){
			this.ajouterArbre(a);
		}
	}
	
	/**
	 * cette mthode enlve le dernier lment de la liste
	 * @return une rfrence sur l'arbre enlev de la liste
	 */
	public ArbreCodage supprimeFin(){
		if(mesArbres.isEmpty())
			return null;
		else {
			ArbreCodage ar = mesArbres.get(mesArbres.size() - 1);
			mesArbres.removeElementAt(mesArbres.size() - 1);
			return ar;
		}
	}
	
	/**
	* @return le premier arbre de la liste
	*/
	public ArbreCodage getDebut(){
		if(mesArbres.isEmpty()) 
			return null;
		else
			return mesArbres.get(0);
	}
	
	/**
	* affiche les poids des diffrents arbres de la liste
	*/
	public void printpoids() {
		for(ArbreCodage ar : mesArbres) {
			System.out.println(ar.getPoids());
		}
	}
	
	
	/** imprime les chanesde caractres associes 
	* aux racines des arbres  de la liste,
	* du premier au dernier
	*/ 
	public void printRacines(){
		for(ArbreCodage ar : mesArbres)
			System.out.println(ar.getLettre());
	}
}

